package com.contouraisdk;

import androidx.annotation.NonNull;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;
import com.contourdocumentimaging.android.contours_ai.ContoursStarterActivity;
import com.contourdocumentimaging.android.contours_ai.callback.IContoursResultListener;
import com.contourdocumentimaging.android.contours_ai.constants.ContoursConstants;
import com.contourdocumentimaging.android.contours_ai.models.ContoursCapturingMode;
import com.contourdocumentimaging.android.contours_ai.models.ContoursModel;
import com.contourdocumentimaging.android.contours_ai.models.ContoursResultModel;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.WritableNativeMap;
import com.facebook.react.module.annotations.ReactModule;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import android.widget.Toast;
import org.json.JSONException;
import org.json.JSONObject;

@ReactModule(name = ContourAiSdkModule.NAME)
public class ContourAiSdkModule extends ReactContextBaseJavaModule {
  public static final String NAME = "ContourAiSdk";
  private Context context;

  public ContourAiSdkModule(ReactApplicationContext reactContext) {
    super(reactContext);
    context = reactContext;
  }

  @ReactMethod
  public void addListener(String eventName) {

  }

  @ReactMethod
  public void removeListeners(Integer count) {

  }

  @Override
  @NonNull
  public String getName() {
    return NAME;
  }

  @ReactMethod
  public void startContourSDK(String captureSide, String clientID, String captureType, boolean enableMultipleCheckCapturing) {
    //Put this code before launching sdk to capture check
    ContoursModel contoursModel = new ContoursModel();
    if(captureType.equalsIgnoreCase("auto")) {
      contoursModel.capturingMode = ContoursCapturingMode.AUTO_CAPTURE;
    } else if (captureType.equalsIgnoreCase("manual")) {
      contoursModel.capturingMode = ContoursCapturingMode.MANUAL_CAPTURE;
    } else {
      contoursModel.capturingMode = ContoursCapturingMode.BOTH_CAPTURE;
    }
    // checkFace value will be either ContoursConstants.FRONT_FACE of ContoursConstants.BACK_FACE as in openContours() function
    if(captureSide.equalsIgnoreCase("front")) {
      contoursModel.checkFace = ContoursConstants.FRONT_FACE;
    } else if (captureSide.equalsIgnoreCase("frontFaceOnly")) {
      contoursModel.checkFace = ContoursConstants.FRONT_FACE_ONLY;
    } else {
      contoursModel.checkFace = ContoursConstants.BACK_FACE;
    }

    contoursModel.enableMultipleCheckCapturing = enableMultipleCheckCapturing;
    ContoursStarterActivity.Companion.launchSdk(context, contoursModel, clientID, new IContoursResultListener() {
      @Override
      public void onCaptureSuccess(ContoursResultModel contoursResultModel) {
         if (contoursResultModel.resultCheckFace.equals(ContoursConstants.FRONT_FACE)) {
           sendDataToHost(contoursResultModel.resultFrontImageUri, contoursResultModel.resultRearImageUri, contoursResultModel.resultFrontCroppedImageUri, contoursResultModel.resultRearCroppedImageUri);
         } else if (contoursResultModel.resultCheckFace.equals(ContoursConstants.FRONT_FACE_ONLY)) {
           sendDataToHost(contoursResultModel.resultFrontImageUri, "", contoursResultModel.resultFrontCroppedImageUri, "");
         } else if (contoursResultModel.resultCheckFace.equals(ContoursConstants.BACK_FACE)) {
           sendDataToHost("", contoursResultModel.resultRearImageUri, "", contoursResultModel.resultRearCroppedImageUri);
         }
      }
      @Override
      public void onEventCapture(String eventJsonString) {
          sendEventToHost(eventJsonString);
      }
      @Override
      public void onContourClosed() {
        getReactApplicationContext().getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                .emit("onContourClose", null);
      }
    });
  }

  private void sendDataToHost(String frontUri, String rearUri, String frontUriCropped, String rearUriCropped) {
    WritableNativeMap map = new WritableNativeMap();
    if(frontUriCropped != null && !frontUriCropped.isEmpty()) {
      compressImage(frontUri);
      map.putString("frontUri", adjustUri(frontUri));
      map.putString("croppedFrontUri", adjustUri(frontUriCropped));
    }
    compressImage(rearUri);
    map.putString("rearUri", adjustUri(rearUri));
    map.putString("croppedRearUri", adjustUri(rearUriCropped));
    getReactApplicationContext().getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
            .emit("checkCaptured", map);
  }

  private void sendEventToHost(String event) {
    getReactApplicationContext().getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                .emit("onEventCaptured", event);
  }

  /**
   * This function creating Base64 for the provided uri
   * @param uri
   * @return
   */
  private String encodeImage(String uri) throws URISyntaxException {
    URI imageURI = new URI(uri);
    BitmapFactory.Options options = new BitmapFactory.Options();
    Bitmap bmp = BitmapFactory.decodeFile(imageURI.getPath(), options);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    bmp.compress(Bitmap.CompressFormat.JPEG,100,baos);
    byte[] b = baos.toByteArray();
    return Base64.encodeToString(b, Base64.NO_WRAP);
  }

  private void compressImage(String uri)  {
    if(uri == null || uri.isEmpty()) {
      return;
    }
    URI imageURI = null;
    try {
        imageURI = new URI(uri);
    } catch( URISyntaxException e) {
      e.printStackTrace();
    }
    if(imageURI == null) {
      return;
    }
    BitmapFactory.Options options = new BitmapFactory.Options();
    Bitmap bmp = BitmapFactory.decodeFile(imageURI.getPath(), options);
    File file = new File(imageURI.getPath());
    if (file.exists()) {
        file.delete();
    }
    writeBitmapToFile(bmp, imageURI.getPath());
  }

  private void writeBitmapToFile(Bitmap bmp, String filename) {
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(filename);
            bmp.compress(Bitmap.CompressFormat.JPEG, 80, out);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

  private String adjustUri(String uri) {
    if(uri == null) {
      return "";
    }
      return "file://"+uri;
  }
}
