import { NativeModules, NativeEventEmitter, Platform } from 'react-native';


const LINKING_ERROR =
  `The package 'contour-ai-sdk' doesn't seem to be linked. Make sure: \n\n` +
  Platform.select({ ios: "- You have run 'pod install'\n", default: '' }) +
  '- You rebuilt the app after installing the package\n' +
  '- You are not using Expo Go\n';

const ContourAiSdk = NativeModules.ContourAiSdk
  ? NativeModules.ContourAiSdk
  : new Proxy(
      {},
      {
        get() {
          throw new Error(LINKING_ERROR);
        },
      }
    );

const eventEmitter = new NativeEventEmitter(NativeModules.ContourAiSdk);

export function startContourSDK(captureSide: string, clientId: string, captureType: string,enableMultipleCheckCapturing: Boolean, callback: (data: any) => void) {
  const count = eventEmitter.listenerCount('checkCaptured');
  if (count > 0) {
    eventEmitter.removeAllListeners('checkCaptured');
  }
  eventEmitter.addListener('checkCaptured', (response: any) => {
    callback(response);
  });
  ContourAiSdk.startContourSDK(captureSide, clientId, captureType, enableMultipleCheckCapturing);
}

export function onContourClosed(callback: () => void) {
  const count = eventEmitter.listenerCount('onContourClose');
  if (count > 0) {
    eventEmitter.removeAllListeners('onContourClose');
  }
  eventEmitter.addListener('onContourClose', () => {
    callback();
  });
}
export function onEventCaptured(callback: (data: any) => void) {
  const count = eventEmitter.listenerCount('onEventCaptured');
  if (count > 0) {
    eventEmitter.removeAllListeners('onEventCaptured');
  }
  eventEmitter.addListener('onEventCaptured', (response: any) => {
    callback(response);
  });
}